/* Copyright (c) 2004 Julien Puydt, aka Snark (julien.puydt@laposte.net)
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file COPYING); if not, write to the
 * Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 ****************************************************************
 */

#include <libxml/tree.h>
#include <libxml/xpath.h>
#include <gconf/gconf-client.h>
#include <stdlib.h>
#include <string.h>

/* the functions that take care of the program's
* exported features 
*/
int dump_from_gconf ();
int load_from_file (char*);

/* the helper functions, to convert from one form
 * of addressbook representation to another, those
 * being:
 * xml: the representation as a text file
 *           (respecting the dtd)
 * ab: the internal representation, that is a libml2 tree
 *         whose children are groups, as described in the dtd
 * gconf: the representation in gnomemeeting v. 1.0, as a set
 *             of various gconf keys.
 */
typedef xmlNodePtr AB;
AB ab_from_xml(char *);
AB ab_from_gconf ();
int gconf_from_ab (AB);
int xml_from_ab (AB);

/*void print_ab(AB) */

#define GROUPS_LIST_KEY		"/apps/gnomemeeting/contacts/groups_list"
#define GROUP_PREFIX_KEY	"/apps/gnomemeeting/contacts/groups/"

int
main(int argc, char *argv[])
{
  /* stupid way of doing things: if the user tries a --help, we fail to give usage! */
  switch (argc)
    {
    case 1: /* only the name: we're dumping the gconf addressbook as an xml-file! */
      return dump_from_gconf ();
      break;
    case 2: /* we have an argument: interpret it as the filename of an xml addressbook to load in gconf */
      return load_from_file (argv[1]);
      break;
    }
  /* fallback: we were given too many arguments... we give the usage! */
  fprintf (stderr, "Usage:\n");
  fprintf (stderr, "%s            dumps addressbook from gconf\n", argv[0]);
  fprintf (stderr, "%s <filename> creates gconf addressbook from xml file\n", argv[0]);
  return EXIT_FAILURE;
}

/* first function of the program, surely the only interesting: take the gnomemeeting v.1.0 addressbook, and
 *  dump it as an xml file, ready for use in a later and even better version !
 */
int
dump_from_gconf ()
{
  AB ab = ab_from_gconf ();
  if (ab)
    return xml_from_ab (ab);
  else
    return EXIT_FAILURE;
}

/* second function of the program: take a gnomemeeting v.1.0 addressbook from an xml file, and load it in
 * gconf, ready to use in gnomemeeting itself.
 * 
 * notice: I had to use it to clean my addressbook, since I corrupted gnomemeeting's addressbook
 * while working on it, and couldn't repair it with gconf-editor, so that was a motivation to write that ;-)
 */
int
load_from_file (char *filename)
{
  fprintf (stderr, "Trying to load from %s\n", filename);
  AB ab = ab_from_xml (filename);
  if (ab)
    {
      return gconf_from_ab (ab);
    } else
      return EXIT_FAILURE;
}

/* IN: any char*
 * OUT: an AB, that is NULL is things failed
 *
 * an internal addressbook is just the libxml2 representation, with the head cut away...
 * so loading it is just loading&cutting.
 */
AB
ab_from_xml (char *filename)
{
  xmlDocPtr tree = NULL;
  tree = xmlParseFile (filename); /* try to load */
  if (!tree)
    {
      fprintf (stderr, "Couldn't load the file!\n");
      return NULL;
    }
	
  return tree->children; /* cut the head */
}

/* IN: nil
    OUT: an addressbook in internal representation
 */
AB
ab_from_gconf ()
{
  AB ab = xmlNewNode (NULL, "addressbook"); /* create the root of the ab */
  xmlNodePtr group = NULL;
  xmlNodePtr contact = NULL;

  GConfClient *client = gconf_client_get_default ();

  GSList *groups = NULL;
  GSList *groups_iter = NULL;
  gchar *group_name;

  gchar *contacts_key = NULL;
  GSList *contacts = NULL;
  GSList *contacts_iter = NULL;

  char **contact_info = NULL;
  gchar *name = NULL;
  gchar *url = NULL;
  gchar *speed_dial = NULL;

  groups = gconf_client_get_list (client,
				  GROUPS_LIST_KEY,
				  GCONF_VALUE_STRING, NULL);
  groups_iter = groups;
  while (groups_iter) /* loop on all the groups in gconf */
    {
      if (groups_iter->data) /* of course, consider only valid ones!*/
	{
	  /* get the name of the group, and prepare the tree node, that we attach in the ab */
	  group_name = gconf_unescape_key ((char *)groups_iter->data, -1);
	  group = xmlNewNode (NULL, "group");
	  xmlSetProp (group, "name", group_name);
	  xmlAddChild (ab, group);
	  /* find where the contacts are stored from the group name, and get the list */
	  contacts_key =  g_strdup_printf ("%s%s", GROUP_PREFIX_KEY,
					   g_utf8_strdown ((char *)groups_iter->data, -1));
	  contacts = gconf_client_get_list (client, contacts_key, GCONF_VALUE_STRING, NULL);
	  contacts_iter = contacts;
	  while (contacts_iter)
	    { /* loop on all the contacts in the group */
	      /* a contact is stored by gnomemeeting as either name|url|speeddial or name|url */
	      contact_info = g_strsplit ((char *)contacts_iter->data, "|", 0);
	      name = contact_info[0];
	      url = contact_info[1];
	      speed_dial = contact_info[2]; /* doesn't always exist */
	      /* create a tree node for the contact, and put below all the information we have*/
	      contact = xmlNewNode (NULL, "contact");
	      xmlNewTextChild (contact, NULL, "name", name);
	      xmlNewTextChild (contact, NULL, "url", url);
	      if (speed_dial)
		xmlNewTextChild (contact, NULL, "speed_dial", speed_dial);
	      /* add the new contact under the group's node */
	      xmlAddChild (group, contact);
 
	      contacts_iter = contacts_iter->next;
	    }
	}
	
      groups_iter = groups_iter->next;
    }

  return ab;
}

/* IN: internal addressbook
 * OUT: success report (side effect)
 * take an internal addressbook, and stick it in gconf
 */
int
gconf_from_ab (AB ab)
{
  GConfClient *client = gconf_client_get_default ();
  
  GSList *groups = NULL;
  for (xmlNodePtr group = ab->children; group; group = group->next)
    { // loop on the nodes of first level: they are the groups
      gchar *group_name = xmlGetProp (group, "name");
      groups = g_slist_append (groups, group_name); /*add it to the string list of groups */
      gchar *contacts_key =  g_strdup_printf ("%s%s", GROUP_PREFIX_KEY,
					      g_utf8_strdown (group_name, -1));
      GSList *contacts = NULL;
      for (xmlNodePtr contact = group->children; contact; contact = contact->next)
	{ // loop on the nodes under the group: they are the contacts in the group
	  if (!strcmp(contact->name, "contact"))
	    {
	      gchar *name = NULL;
	      gchar *url = NULL;
	      gchar *speed_dial = NULL;
	      for (xmlNodePtr info = contact->children; info; info = info->next)
		{ /* loop on the nodes under the contact: they are the data of the contact*/
		  if (info->type == XML_ELEMENT_NODE)
		    {
		      if (!strcmp (info->name, "name"))
			name = info->children->content;
		      else if (!strcmp (info->name, "url"))
			url = info->children->content;
		      else if (!strcmp (info->name, "speed_dial"))
			speed_dial = info->children->content;
		    }
		}
	      if (name && url) /* should always be the case, but I preferred to play it safe */
		{
		  /* prepare the string that describes the contact*/
		  char *contact_info = NULL;
		  if (speed_dial)
		    contact_info = g_strdup_printf ("%s|%s|%s", name, url, speed_dial);
		  else
		    contact_info = g_strdup_printf ("%s|%s", name, url);
		  contacts = g_slist_append (contacts, contact_info); /* add the contact to the list of contacts in that group*/
		}
	    }
	} /* loop on contacts */
      gconf_client_set_list (client, contacts_key, GCONF_VALUE_STRING, contacts, NULL); /* store the contacts' representations in the convenient key */
    } /* loop on groups */
  gconf_client_set_list (client, GROUPS_LIST_KEY, GCONF_VALUE_STRING, groups, NULL); /* we saw all groups, we can store the list*/
  return 0;
}

/* IN: internal addressbook
 * OUT: success report (side effect)
 */
int
xml_from_ab (AB ab)
{
  /* put a head to the addressbook, and use the normal dump method! */
  xmlDocPtr doc = xmlNewDoc ("1.0");
  xmlAddChild ((xmlNodePtr)doc, ab);
  xmlDocDump (stdout, doc);
  return 0;
}


/* unused here, but has proven useful during debugging! */
void
print_ab (AB ab)
{
  for (xmlNodePtr group = ab->children; group; group = group->next)
    {
      fprintf (stderr, "Group name: %s\n", xmlGetProp(group, "name"));
      for (xmlNodePtr contact = group->children; contact; contact = contact->next)
	{
	  if (!strcmp(contact->name, "contact"))
	    {
	      fprintf (stderr, "  contact\n");
	      for (xmlNodePtr info = contact->children; info; info = info->next)
		{
		  if (info->type == XML_ELEMENT_NODE)
		    {
		      fprintf (stderr, "    %s: %s\n", info->name,
			       (info->children->content != NULL)?(char*)info->children->content:"(null)");
		    }
		}
	    }
	}
    }
}

